/*! \mainpage QuickFIX Documentation
* <b>Installation</b>
* <ul>
* <li><a href="dependencies.html">Dependencies</a></li>
* <li><a href="building.html">Building</a></li>
* <li><a href="generating_databases.html">Generating Databases</a></li>
* <li><a href="testing.html">Testing</a></li>
* </ul>
* <b>Getting Started</b>
* <ul>
* <li><a href="project.html">Setting Up Your Project</a></li>
* <li><a href="application.html">Creating Your Application</a></li>
* <li><a href="configuration.html">Configuration</a></li>
* <li><a href="xmlschema.html">Validation</a></li>
* </ul>
* <b>Working With Messages</b>
* <ul>
* <li><a href="receiving_messages.html">Receiving Messages</a></li>
* <li><a href="sending_messages.html">Sending Messages</a></li>
* <li><a href="repeating_groups.html">Repeating Groups</a></li>
* <li><a href="user_defined.html">User Defined Fields</a></li>
* <li><a href="examples.html">Example Applications</a></li>
* </ul>
* <b>Testing</b>
* <ul>
* <li><a href="unit_tests.html">Unit Tests</a></li>
* <li><a href="acceptance_tests.html">Acceptance Tests</a></li>
* </ul>
*
*The Financial Information Exchange (<a href="http://www.fixprotocol.org">FIX</a>) Protocol is a message standard developed
*to facilitate the electronic exchange of information related to securities transactions.
*It is intended for use between trading partners wishing to automate communications.
*
*<I>QuickFIX is a free and open source implementation of the <a href="http://www.fixprotocol.org">FIX</a> protocol.</I>
*
* \author Oren Miller oren@quickfixengine.org
*/
