#ifndef CONFIG_H_IN
#define CONFIG_H_IN

/* #undef HAVE_EMX */
#define HAVE_CXX17
#define HAVE_STD_SHARED_PTR
/* #undef HAVE_SHARED_PTR_IN_TR1_NAMESPACE */
/* #undef HAVE_STD_TR1_SHARED_PTR_FROM_TR1_MEMORY_HEADER */
#define HAVE_STD_UNIQUE_PTR

#endif
